// LED "Firmware" for HELIOS/Transputer load-display
// v0.3 - Added two colors in one row for CPU-load and Mem-Usage
// v0.4 - Removed interupt for QVAL - The CPU is fast enough for polling.
// v0.5 - added handling of >32 CPUs & DisplayLoadOnly

#include <ht1632c.h>
#include <Wire.h>

// 8574  Address range is 0x20-0x27
#define INaddr 0x20  // 8574 addr 000
#define OUTaddr 0x21 // 8574 addr 001
#define QVAL 2
#define QACK 3

byte C011_data;
boolean Command = false;
boolean Tset = false;
boolean TgotLoad = false;
boolean DisplayLoadOnly = false;
int LEDColumns = 1, LEDleds = 16;
int LEDRefresh = 3, loops = 0;      // counting #'o loops for LED refresh
float LEDfactor = 0.16;
int Transputer;
int Tload, Tmem; 

  
ht1632c ledMatrix = ht1632c(&PORTD, 7, 6, 5, 4, GEOM_32x16, 2);

void setup() {
  // ---- Debugging ---- 
  Serial.begin(9600);
  
  //  ---- I2C / Inmos C011 ---- 
  // On most Arduino boards, SDA (data line) is on analog input pin 4
  // and SCL (clock line) is on analog input pin 5
  pinMode(QVAL, INPUT); // Pin 2, QVAL - Goes HIGH if valid data available at C011 Output
                        // So NO pull-up needed!
  pinMode(QACK, OUTPUT); // Pin 3, QAck - Set HIGH to acknowledge the successful read
  //#define TWBR=400000L // This speeds i2c from 100 to 400kHz
  Wire.begin(); 
  expanderSetInput(INaddr, 0xFF);
  
  //  ---- LEDs ---- 
  ledMatrix.clear();  
  ledMatrix.pwm(7);
}

void loop() {
  
  int row, origin; // row and "Nullpunkt" to draw the bar

  if (digitalRead(QVAL) == HIGH) {
    C011_data = expanderRead(INaddr);
    
    if (Command || Tset) { // Command flag set, so this byte is a command
      if (Command) {
        switch (C011_data) {
          case 1:  // Set the display brightness
          case 2:
          case 3:
          case 4:
          case 5:
          case 6:
          case 7:
          case 8:
          case 9:
          case 10:
          case 11:
          case 12:
          case 13:
          case 14:
          case 15: // Max. bright
            ledMatrix.pwm(C011_data);
            break;
          case 31:
            LEDColumns = 1; // max. 32 CPUs, 1 per row, 16 LEDs per CPU
            LEDfactor = 0.15;
            LEDleds = 15;  // Zero-indexed
            break;            
          case 32:
            LEDColumns = 2; // max. 64 CPUs, 2 per row, 8 LEDs per CPU
            LEDfactor = 0.07;
            LEDleds = 7;  // Zero-indexed
            break;
          case 33:
            LEDColumns = 3; // max. 96 CPUs, 3 per row, 5 LEDs per CPU
            LEDfactor = 0.04;
            LEDleds = 4;  // Zero-indexed            
            break;
          case 34:
            LEDColumns = 4; // max. 128 CPUs, 4 per row, 4 LEDs per CPU
            LEDfactor = 0.03;
            LEDleds = 3;  // Zero-indexed            
            break;            
          case 99:
            ledMatrix.clear(); 
            break;
          case 100:
            DisplayLoadOnly = !DisplayLoadOnly; // toggle DisplayLoadOnly flag
            break;
          case 101:
          case 102:
          case 103:
          case 104:
          case 105:
          case 106:
            LEDRefresh = 8 * (C011_data - 100);
            break;
          // There's no 'default:' here
        }    
        Serial.println(C011_data);
        Command = false;
      }
    
      if (Tset) { // Transputer set...
        if (! TgotLoad) { // so the 2nd byte is the load-value
         Tload = C011_data * LEDfactor; // was: LEDleds - (C011_data * LEDfactor);
         // e.g. load 66% 
         // 16 LEDs per line = 66 * 0.16 = 10,56 =~ 11 LEDs
         //  8 LEDs per line = 66 * 0.08 = 5,28 =~ 5 LEDs
         TgotLoad = true;
        } else { // ...and the 3rd byte is the mem-value, so we can draw now:
        
        // Calculate the row(y) and origin(x) for each bar 
        if (Transputer <= 31) { 	
            row = Transputer;
            origin = 15;  
        } else if ((Transputer > 31) && (Transputer <= 63)) {
            row = Transputer - 32; // wrap to zero
            if (LEDColumns == 2) {
              origin = 7; 	
            } else if (LEDColumns == 3) {
              origin = 10;
            } else if (LEDColumns == 4) {
              origin = 11;
            }
        } else if ((Transputer > 63) && (Transputer <= 95)) {
            row = Transputer - 64;	
            if (LEDColumns == 3) {
              origin = 5; 	
            } else if (LEDColumns == 4) {
              origin = 7;
            }         
        } else if ((Transputer > 95) && (Transputer <= 127)) {
            row = Transputer - 96;	 	
            origin = 3;
        }

        Tmem = origin - (C011_data * LEDfactor);
        Tload = origin - Tload;
        
         ledMatrix.line(row, origin, row, origin - LEDleds, BLACK); // first clear the whole line
         if (!DisplayLoadOnly) {
           if (Tload < Tmem) { // Logic reversed b/c of conversion below
           // a bit of conversion here to have LED 0,16 as origin
            ledMatrix.line(row, Tload, row, origin, RED);
            ledMatrix.line(row, Tmem, row, origin, ORANGE);
           } else if (Tload > Tmem) {
            ledMatrix.line(row, Tmem, row, origin, GREEN);
            ledMatrix.line(row, Tload, row, origin, ORANGE);
           } else if (Tload == Tmem) {
            ledMatrix.line(row, Tload, row, origin, ORANGE);
           }
         } else {
           ledMatrix.line(row, Tload, row, origin, RED);
         }

         // ledMatrix.sendframe();  // been put into main-loop
         Tset = false;
         TgotLoad = false;
        } // end if TgotLoad
      } 
    } else { // Neither Tset nor Command set
        if (C011_data == 255) {
          // 255/0xFF is the "command-flag" so the 2nd byte defines the command
          Command = true;
          Serial.print("Command: ");
        } else {
          // < 255 means "Transputer Number inByte"
          Transputer = C011_data; 
          Tset = true;
          Serial.print("Transputer: ");
          Serial.println(Transputer, DEC);
        }
    }
    toggleQACK();
  } // end-if QVAL = HIGH
  
  loops++;
  if (loops == LEDRefresh) {
    ledMatrix.sendframe();
    loops = 0;
  }
}

void toggleQACK() {
  // If this doesn't work we'll need an FALLING interrupt
  digitalWrite(QACK, HIGH);
  while (digitalRead(QVAL) == HIGH) {}; // wait for QVAL going LOW again
  digitalWrite(QACK, LOW);
}

// I2C routines to talk to 8574 and 8574A
void expanderSetInput(int i2caddr, byte dir) {
  Wire.beginTransmission(i2caddr);
  Wire.write(dir);  // outputs high for input
  Wire.endTransmission();    
}

byte expanderRead(int i2caddr) {
  int _data = -1;
  Wire.requestFrom(i2caddr, 1);
  if(Wire.available()) {
    _data = Wire.read();
  }
  return _data;
}




